@@ -578,9 +578,9 @@ meta_screen_cast_stream_src_finalize (GObject *object)
   if (meta_screen_cast_stream_src_is_enabled (src))
     meta_screen_cast_stream_src_disable (src);
 
-  g_clear_pointer (&priv->pipewire_stream, (GDestroyNotify) pw_stream_destroy);
-  g_clear_pointer (&priv->pipewire_remote, (GDestroyNotify) pw_remote_destroy);
-  g_clear_pointer (&priv->pipewire_core, (GDestroyNotify) pw_core_destroy);
+  g_clear_pointer (&priv->pipewire_stream, pw_stream_destroy);
+  g_clear_pointer (&priv->pipewire_remote, pw_remote_destroy);
+  g_clear_pointer (&priv->pipewire_core, pw_core_destroy);
   g_source_destroy (&priv->pipewire_source->base);
 
   G_OBJECT_CLASS (meta_screen_cast_stream_src_parent_class)->finalize (object);
