@@ -50,19 +50,23 @@ meta_wayland_actor_surface_constructed (GObject *object)
 }
 
 static void
-meta_wayland_actor_surface_finalize (GObject *object)
+meta_wayland_actor_surface_dispose (GObject *object)
 {
   MetaWaylandActorSurfacePrivate *priv =
     meta_wayland_actor_surface_get_instance_private (META_WAYLAND_ACTOR_SURFACE (object));
   MetaWaylandSurface *surface =
     meta_wayland_surface_role_get_surface (META_WAYLAND_SURFACE_ROLE (object));
 
-  g_signal_handlers_disconnect_by_func (priv->actor,
-                                        meta_wayland_surface_notify_geometry_changed,
-                                        surface);
-  g_object_unref (priv->actor);
+  if (priv->actor)
+    {
+      g_signal_handlers_disconnect_by_func (priv->actor,
+                                            meta_wayland_surface_notify_geometry_changed,
+                                            surface);
+      clutter_actor_set_reactive (CLUTTER_ACTOR (priv->actor), FALSE);
+      g_clear_object (&priv->actor);
+    }
 
-  G_OBJECT_CLASS (meta_wayland_actor_surface_parent_class)->finalize (object);
+  G_OBJECT_CLASS (meta_wayland_actor_surface_parent_class)->dispose (object);
 }
 
 static void
@@ -72,10 +76,18 @@ meta_wayland_actor_surface_assigned (MetaWaylandSurfaceRole *surface_role)
     meta_wayland_actor_surface_get_instance_private (META_WAYLAND_ACTOR_SURFACE (surface_role));
   MetaWaylandSurface *surface =
     meta_wayland_surface_role_get_surface (surface_role);
+  GList *l;
 
   meta_surface_actor_wayland_add_frame_callbacks (META_SURFACE_ACTOR_WAYLAND (priv->actor),
                                                   &surface->pending_frame_callback_list);
   wl_list_init (&surface->pending_frame_callback_list);
+
+  for (l = surface->subsurfaces; l; l = l->next)
+    {
+      ClutterActor *subsurface_actor =
+        CLUTTER_ACTOR (meta_wayland_surface_get_actor (l->data));
+      clutter_actor_add_child (CLUTTER_ACTOR (priv->actor), subsurface_actor);
+    }
 }
 
 static void
@@ -262,7 +274,7 @@ meta_wayland_actor_surface_class_init (MetaWaylandActorSurfaceClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->constructed = meta_wayland_actor_surface_constructed;
-  object_class->finalize = meta_wayland_actor_surface_finalize;
+  object_class->dispose = meta_wayland_actor_surface_dispose;
 
   surface_role_class->assigned = meta_wayland_actor_surface_assigned;
   surface_role_class->commit = meta_wayland_actor_surface_commit;
