@@ -47,6 +47,9 @@
 #include "meta-wayland-inhibit-shortcuts-dialog.h"
 #include "meta-xwayland-grab-keyboard.h"
 #include "meta-xwayland.h"
+#include "meta-wayland-egl-stream.h"
+
+#include "main-private.h"
 
 static MetaWaylandCompositor _meta_wayland_compositor;
 static char *_display_name_override;
@@ -397,8 +400,13 @@ meta_wayland_init (void)
                                   meta_xwayland_global_filter,
                                   compositor);
 
-  if (!meta_xwayland_start (&compositor->xwayland_manager, compositor->wayland_display))
-    g_error ("Failed to start X Wayland");
+  meta_wayland_eglstream_controller_init (compositor);
+
+  if (meta_should_autostart_x11_display ())
+    {
+      if (!meta_xwayland_start (&compositor->xwayland_manager, compositor->wayland_display))
+        g_error ("Failed to start X Wayland");
+    }
 
   if (_display_name_override)
     {
@@ -419,7 +427,9 @@ meta_wayland_init (void)
       compositor->display_name = g_strdup (display_name);
     }
 
-  set_gnome_env ("DISPLAY", meta_wayland_get_xwayland_display_name (compositor));
+  if (meta_should_autostart_x11_display ())
+    set_gnome_env ("DISPLAY", meta_wayland_get_xwayland_display_name (compositor));
+
   set_gnome_env ("WAYLAND_DISPLAY", meta_wayland_get_wayland_display_name (compositor));
 }
 
